using System;
using System.Threading;
using System.Drawing;
using System.Windows.Forms;

class MBView : Panel
{
	// Members
	private Mandelbrot mb;
	private char waitSign;
	private bool wait;
	private ProgressBar pb;
	private int p;
	private Point mouseOver;
	private Point mouseStart;
	private Rectangle mouseRect;
	private bool dragRect;
	private double xMin;
	private double xMax;
	private double yMin;
	private double yMax;
	private double xUnit;
	private double yUnit;
	
	// C'tor
	public MBView(Mandelbrot mb)
	{
		this.mb = mb;
		this.waitSign = (char)0;
		this.wait = false;
		this.pb = new ProgressBar();
		this.pb.Width = 150;
		this.pb.Visible = false;
		this.Controls.Add(this.pb);
		this.mouseOver = new Point();
		this.mouseStart = new Point();
		this.mouseRect = new Rectangle();
		this.dragRect = false;
		// this.SetStyle(ControlStyles.DoubleBuffer, true);		// .net 1.1
		this.DoubleBuffered = true;								// .net 2.0
	}
	
	// Getter
	// Positions from the MouseRect converted to values
	public double XMin
	{
		get 
		{
			if (!this.mouseStart.IsEmpty)
				return this.xMin + this.xUnit * this.mouseRect.Left;
			else
				return 0;
		}
	}
	
	public double XMax
	{
		get
		{
			if (!this.mouseStart.IsEmpty)
				return this.xMin + this.xUnit * this.mouseRect.Right;
			else
				return 0;
		}
	}
	
	public double YMin
	{
		get 
		{
			if (!this.mouseStart.IsEmpty)
				return this.yMax - this.yUnit * this.mouseRect.Bottom;
			else
				return 0;
		}
	}

	public double YMax
	{
		get
		{
			if (!this.mouseStart.IsEmpty)
				return this.yMax - this.yUnit * this.mouseRect.Top;
			else
				return 0;
		}
	}
	
	// Methods
	public void setSizeValues(double xMin, double xMax, double yMin, double yMax)
	{
		this.xMin = xMin;
		this.xMax = xMax;
		this.yMin = yMin;
		this.yMax = yMax;

		if (this.xMax > 0)
			this.xUnit = this.xMax - this.xMin;
		else
			this.xUnit = Math.Abs(this.xMin) + this.xMax;
		this.xUnit /= this.ClientSize.Width;

		if (this.yMax > 0)
			this.yUnit = this.yMax - this.yMin;
		else
			this.yUnit = Math.Abs(this.yMin) + this.yMax;
		this.yUnit /= this.ClientSize.Height;
	}
	
	public void Wait()
	{
		this.wait = true;
		this.Cursor = Cursors.WaitCursor;
		this.pb.Visible = true;
		ThreadStart ts = new ThreadStart(this.WaitWorker);
		Thread t = new Thread(ts);
		t.IsBackground = true;
		t.Start();
	}
	
	private char NextWaitSign()
	{
		switch (this.waitSign)
		{
			case (char)0:
			case '\\':
				return '|';
			case '|':
				return '/';
			case '/':
				return '-';
			case '-':
				return '\\';
			default:
				return (char)0;
		}
	}
	private void WaitWorker()
	{
		while (this.wait)
		{
			this.waitSign = this.NextWaitSign();
			this.Invalidate();
			Thread.Sleep(100);
		}
	}
	
	public void StopWaiting()
	{
		this.wait = false;
		MandelbrotCompleteHandler h = new MandelbrotCompleteHandler(this.ResetControls);
		this.Invoke(h);
		this.Invalidate();
	}
	
	private void ResetControls()
	{
		this.pb.Visible = false;
		this.mouseStart.X = this.mouseStart.Y = 0;
		this.Cursor = Cursors.Default;
	}
	
	public void ListenProgress(int percentage)
	{
		MandelbrotProceedHandler h = new MandelbrotProceedHandler(this.touchProgressBar);
		this.Invoke(h, new object[] {percentage});
	}
	
	private void touchProgressBar(int percentage)
	{
		this.pb.Value = percentage;
		// this.p = percentage;
	}

	public void DrawMandelbrot()
	{
		this.StopWaiting();
		this.Invalidate();
	}
	
	private string getValuesFromCoords(Point p)
	{
		return (this.xMin + this.xUnit * p.X).ToString() + " + i" + (this.yMin + this.yUnit * p.Y).ToString(); ;
	}

	protected override void OnPaint(PaintEventArgs e)
	{
		base.OnPaint(e);

		this.pb.Top = this.Height / 2 - this.pb.Height / 2;
		this.pb.Left = this.Width / 2 - this.pb.Width / 2;

		//if (!this.mb.IsWorking)
			e.Graphics.DrawImage(this.mb.Bitmap, this.ClientRectangle);
			
		e.Graphics.DrawString(getValuesFromCoords(this.mouseOver), this.Font, Brushes.WhiteSmoke, this.mouseOver.X + 16, this.mouseOver.Y);
		if (!this.mouseStart.IsEmpty)
			e.Graphics.DrawRectangle(Pens.LightSteelBlue, this.mouseRect);
	}

	protected override void OnMouseDown(MouseEventArgs e)
	{
		base.OnMouseDown(e);

		this.dragRect = true;
		this.mouseStart.X = e.X;
		this.mouseStart.Y = e.Y;
	}
	
	protected override void OnMouseMove(MouseEventArgs e)
	{
		base.OnMouseMove(e);
		
		this.mouseOver.X = e.X;
		this.mouseOver.Y = e.Y;
		
		if (this.dragRect)
		{
			// drag down + right
			if (this.mouseOver.X > this.mouseStart.X && this.mouseOver.Y > this.mouseStart.Y)
			{
				this.mouseRect.Location = this.mouseStart;
				this.mouseRect.Width = this.mouseOver.X - this.mouseStart.X;
				this.mouseRect.Height = this.mouseOver.Y - this.mouseStart.Y;
			}

			// drag up + left
			if (this.mouseOver.X < this.mouseStart.X && this.mouseOver.Y < this.mouseStart.Y)
			{
				this.mouseRect.Location = this.mouseOver;
				this.mouseRect.Width = this.mouseStart.X - this.mouseOver.X;
				this.mouseRect.Height = this.mouseStart.Y - this.mouseOver.Y;
			}

			// drag up + right
			if (this.mouseOver.X > this.mouseStart.X && this.mouseOver.Y < this.mouseStart.Y)
			{
				this.mouseRect.X = this.mouseStart.X;
				this.mouseRect.Y = this.mouseOver.Y;
				this.mouseRect.Width = this.mouseOver.X - this.mouseStart.X;
				this.mouseRect.Height = this.mouseStart.Y - this.mouseOver.Y;
			}

			// drag down + left
			if (this.mouseOver.X < this.mouseStart.X && this.mouseOver.Y > this.mouseStart.Y)
			{
				this.mouseRect.X = this.mouseOver.X;
				this.mouseRect.Y = this.mouseStart.Y;
				this.mouseRect.Width = this.mouseStart.X - this.mouseOver.X;
				this.mouseRect.Height = this.mouseOver.Y - this.mouseStart.Y;
			}			
		}
		
		this.Invalidate();
	}

	protected override void OnMouseUp(MouseEventArgs e)
	{
		base.OnMouseUp(e);

		if (this.mouseStart == e.Location)
			this.mouseStart.X = this.mouseStart.Y = 0;
		this.dragRect = false;
	}
}