using System;
using System.Threading;
using System.Drawing;

public delegate void MandelbrotCompleteHandler();
public delegate void MandelbrotProceedHandler(int percentage);

struct MandelbrotInfo
{
	public int Depth;
	public double XMin;
	public double XMax;
	public double YMin;
	public double YMax;
	public Size Size;
}

class Mandelbrot
{
	// Members
	private bool isRunning;
	private MandelbrotInfo mi;
	public event MandelbrotCompleteHandler MandelbrotComplete;
	public event MandelbrotProceedHandler MandelbrotProceed;
	private int currentPercentage;
	private Bitmap publicBmp;
	private Bitmap privateBmp;
	
	// C'tor
	public Mandelbrot()
	{
		this.isRunning = false;
		this.publicBmp = new Bitmap(1, 1);
	}
	
	// Getter
	public bool IsWorking
	{
		get {return this.isRunning;}
	}

	public Bitmap Bitmap
	{
		get {return this.publicBmp;}
	}
	
	// Methods
	public static MandelbrotInfo CreateInfo(int depth, double xMin, double xMax, double yMin, double yMax, Size s)
	{
		MandelbrotInfo i;
		i.Depth = depth;
		i.XMin = xMin;
		i.XMax = xMax;
		i.YMin = yMin;
		i.YMax = yMax;
		i.Size = s;
		return i;
	}
	
	public void Start(MandelbrotInfo mi)
	{
		this.isRunning = true;
		this.mi = mi;
		this.currentPercentage = 0;
		ThreadStart ts = new ThreadStart(this.Calculate);
		Thread t = new Thread(ts);
		t.IsBackground = true;
		t.Start();
	}

	public void Stop()
	{
		this.isRunning = false;
	}
	
	private int Sequence(ComplexNr z)
	{
		int  nrOfSteps = 0;
		ComplexNr c = new ComplexNr(z.Re, z.Im);
		
		while (z.Abs < 2.0 && nrOfSteps < this.mi.Depth)
		{
			z = z * z + c;
			nrOfSteps++;
		}

		return (nrOfSteps < this.mi.Depth) ? nrOfSteps : (nrOfSteps-1);
	}
	
	private void Calculate()
	{
		if (this.privateBmp != null)
			this.privateBmp.Dispose();

		this.privateBmp = new Bitmap(this.mi.Size.Width, this.mi.Size.Height);
		
		Color[] c = new Color[this.mi.Depth];
		int colRange = this.mi.Depth / 3;
		int colStep = 255 / colRange;
		
		for (int i=0; i<this.mi.Depth; i++)
		{
			int r = (i < colRange) ? (i * colStep) : 255;
			int g = (i > colRange && i < colRange * 2) ? ((i % colRange) * colStep) : (i < colRange) ? 0 : 255;
			int b = (i > colRange * 2) ? ((i % colRange) * colStep) : 0;
			c[i] = Color.FromArgb(r, g, b);
		}
				
		double xUnit;
		double yUnit;
		
		if (this.mi.XMax > 0)
			xUnit = this.mi.XMax - this.mi.XMin;
		else
			xUnit = Math.Abs(this.mi.XMin) + this.mi.XMax;
		xUnit /= this.mi.Size.Width;

		if (this.mi.YMax > 0)
			yUnit = this.mi.YMax - this.mi.YMin;
		else
			yUnit = Math.Abs(this.mi.YMin) + this.mi.YMax;
		yUnit /= this.mi.Size.Height;
		
		for (int x=0; x<this.mi.Size.Width; x++)
		{
			for (int y=0; y<this.mi.Size.Height; y++)
			{
				double re = this.mi.XMin + (double)x * xUnit;
				double im = this.mi.YMax - (double)y * yUnit;
				ComplexNr z = new ComplexNr(re, im);
				
				int nrOfSteps = this.Sequence(z);

				if (!this.isRunning)
				{
					Bitmap tmpBmp0 = (Bitmap)this.privateBmp.Clone();
					this.publicBmp = tmpBmp0;
					
					return;
				}

				if (nrOfSteps >= this.mi.Depth - 5)
					this.privateBmp.SetPixel(x, y, Color.Black);
				else
					this.privateBmp.SetPixel(x, y, c[nrOfSteps]);

				//if (z.Abs < 2.0)
				//    this.Bitmap.SetPixel(x, y, c[nrOfSteps]);
				//else
				//    this.Bitmap.SetPixel(x, y, Color.Black);

				int nrOfPixels = (((x * this.mi.Size.Height + y) * 100) / (this.mi.Size.Width * this.mi.Size.Height));
				if (this.currentPercentage < nrOfPixels)
				{
					this.currentPercentage = nrOfPixels;
					if (MandelbrotProceed != null)
						MandelbrotProceed(this.currentPercentage);
				}
			}
		}

		Bitmap tmpBmp1 = (Bitmap)this.privateBmp.Clone();
		this.publicBmp = tmpBmp1;
		
		this.isRunning = false;
		
		if (this.MandelbrotComplete != null)
			this.MandelbrotComplete();
	}
}